import { world, system } from "@minecraft/server";
import {
    getScore, setScore, modifyScore, clearBombFromInventory,
    playSound, Tellraw, hasItemInInventory
} from "../other/utils";
import { runcmd, runcmd_at } from "../other/utils";
import { stopTimerSpecialForces } from "../game/special_forces_timer";
import { startTimerExplode } from "../game/explodeTimer";
import { hasItemInHand, ckeckItemInHand} from '../game/itemsInHand';
import { overGame} from '../game/overGame';

const bombTimer = new Map();
const playerStartPositions = new Map();
const BOMB_TIME = 55;
const MOVEMENT_THRESHOLD = 1;

world.beforeEvents.itemUse.subscribe((event) => {
    const player = event.source;

    if (event.itemStack.typeId !== "fear:bomb") return;
    if (getScore("fear:settings", "bomb_condition", true) === 1) return;
    if (bombTimer.has(player.id)) return;

    isPlayerEligibleToPlant(player, false, false).then((eligible) => {
        if (eligible) {
            if (!playerStartPositions.has(player.id)) {
                playerStartPositions.set(player.id, { ...player.location });
            }
            startBombTimer(player, false);
        }
    });
});


function isPlayerEligibleToPlant(player, checkSneak = true, checkHand = false) {
    return new Promise((resolve) => {
        const bombStatus = getScore("fear:settings", "bomb_condition", true);
        if (bombStatus === 1) return resolve(false);

        if (!hasItemInInventory(player, "fear:bomb")) return resolve(false);

        const blockUnder = world.getDimension("overworld").getBlock({
            x: Math.floor(player.location.x),
            y: Math.floor(player.location.y) - 1,
            z: Math.floor(player.location.z),
        });

        if (!blockUnder || blockUnder.typeId !== "minecraft:red_wool") return resolve(false);
        if (checkSneak && !player.isSneaking) return resolve(false);

        if (playerStartPositions.has(player.id)) {
            const startPos = playerStartPositions.get(player.id);
            const dx = Math.abs(player.location.x - startPos.x);
            const dz = Math.abs(player.location.z - startPos.z);
            if (dx > MOVEMENT_THRESHOLD || dz > MOVEMENT_THRESHOLD) return resolve(false);
        }

        if (checkHand) {
            ckeckItemInHand(player)
                .then(() => {
                    const foundItem = hasItemInHand(player);
                    return resolve(foundItem === "fear:bomb");
                })
                .catch(() => resolve(false));
        } else {
            return resolve(true);
        }
    });
}


function startBombTimer(player, isSneakMode = true) {
    if (bombTimer.has(player.id)) return;

    bombTimer.set(player.id, 0);
    if (!playerStartPositions.has(player.id)) {
        playerStartPositions.set(player.id, { ...player.location });
    }

    const name = player.name;
    runcmd_at(name, `execute as @s run playanimation @s animation.player.press_button`, true);
    world.getDimension("overworld").playSound("fear.bomb_initiate", player.location);

    function tick() {
        if (!bombTimer.has(player.id)) return;

        const currentTime = bombTimer.get(player.id);
        bombTimer.set(player.id, currentTime + 1);

        isPlayerEligibleToPlant(player, isSneakMode).then((eligible) => {
            if (!eligible) {
                resetBombTimer(player);
            } else if (currentTime >= BOMB_TIME) {
                plantBomb(player);
            } else {
                system.runTimeout(tick, 1);
            }
        });
    }

    tick();
}

function resetBombTimer(player) {
    bombTimer.delete(player.id);
    playerStartPositions.delete(player.id);
}

function plantBomb(player) {
    const bomb_protection = getScore('fear:settings', 'bomb_protection', true);
    bombTimer.delete(player.id);
    playerStartPositions.delete(player.id);

    setScore("fear:settings", "bomb_condition", 1);
    setScore("fear:settings", "startTimerExplode", 45);

    world.getDimension("overworld").playSound("fear.bomb_activated", player.location);
    clearBombFromInventory(player);
    world.getDimension("overworld").spawnEntity("fear:entites_bomb", player.location);

    Tellraw(player, `+300$ §a`, "reward.plant");
    modifyScore("fear:balance", player.name, 300);

    if (bomb_protection === 1) {
        console.warn("plantBomb конец раунда, запуск отменён");
        return; 
    }

    stopTimerSpecialForces();
    system.runTimeout(() => {
        startTimerExplode();
    }, 3);
}

system.runInterval(() => {
    for (const player of world.getAllPlayers()) {
        if (bombTimer.has(player.id)) continue;

        isPlayerEligibleToPlant(player, true, true).then((eligible) => {
            if (eligible) {
                if (!playerStartPositions.has(player.id)) {
                    playerStartPositions.set(player.id, { ...player.location });
                }
                startBombTimer(player, true);
            }
        });

    }
    const terrorists = world.getPlayers().filter(p => p.hasTag("game:terrorist"));
    const commandos = world.getPlayers().filter(p => p.hasTag("game:commando"));
    
    const gameStatus = getScore('fear:settings', 'gameStatus', true);
    if (gameStatus === 1) {
        if (terrorists.length === 0 && commandos.length === 0) {
            const overGameStatus = 0;
            overGame(overGameStatus); 
            return; 
        }
    }

}, 5);
