import { world, system } from "@minecraft/server";
import { getPlayerTeam, getColoredName, getPlayerByName, distance } from "./grenades.js";
import {  Tellraw, modifyScore, runcmd } from '../../other/utils.js';
import {  updateTagData } from '../../other/tagUtils.js';

const activeFireGrenades = new Map();

world.beforeEvents.itemUse.subscribe(event => {
    const { itemStack, source } = event;
    
    if (itemStack.typeId === "addon:fire_grenade" && source.typeId === "minecraft:player") {
        activeFireGrenades.set(source.id, {
            name: source.name,
            team: getPlayerTeam(source),
            location: { ...source.location }
        });
    }
});

export function handleFireGrenade(entity) {
    const thrower = getNearestThrower(entity.location);
    if (!thrower) return;

    const checkFall = () => {
        if (!entity || entity.isRemoved) return; 
        if (entity.getVelocity().y === 0) {
            system.runTimeout(() => igniteFireGrenade(entity, thrower.name, thrower.team), 30);
        } else {
            system.run(checkFall); 
        }
    };

    system.run(checkFall); 
    activeFireGrenades.delete(thrower.id);
}


function getNearestThrower(grenadePos) {
    let nearest = null;
    let minDist = 4;

    for (let [id, thrower] of activeFireGrenades.entries()) {
        let dist = distance(grenadePos, thrower.location);
        if (dist < minDist) {
            minDist = dist;
            nearest = { id, ...thrower };
        }
    }
    return nearest;
}

function igniteFireGrenade(grenade, throwerName, throwerTeam) {
    if (!grenade || grenade.isRemoved) return;

    world.getDimension("overworld").playSound("addon.fire_grenade_explode", grenade.location);
    world.getDimension("overworld").playSound("addon.fire_grenade_ignite", grenade.location);
    

    runcmd(`execute as @e[type=addon:fire_grenade] at @s run event entity @s addon:fire`, true);

    grenade.addTag("game:grenadeFireTrue");

    let lifeTime = 200;
    let tickCounter = 0;

    function burnArea() {
        if (!grenade || grenade.isRemoved) return;
        if (tickCounter >= lifeTime) {
            grenade.remove();
            return;
        }

        world.getDimension("overworld").spawnParticle("addon:fire_grenade", grenade.location);
        world.getDimension("overworld").spawnParticle("addon:fire_grenade2", grenade.location);

        let victims = [];
        let thrower = getPlayerByName(throwerName);
        if (!thrower) return; 

        let selfKill = false;
        const coloredThrowerName = getColoredName(throwerName, throwerTeam);

        for (let player of world.getAllPlayers()) {
            let playerTeam = getPlayerTeam(player);
            let dist = distance(grenade.location, player.location);

            if (dist <= 4.5) {
                player.applyDamage(4);

                if (player.getComponent("minecraft:health")?.currentValue <= 0) {
                    victims.push(player);
                    if (player.name === throwerName) selfKill = true;
                }
            }
        }

        const playersInGame = world.getPlayers().filter(playerGame => playerGame.hasTag("fear:game"));

        if (selfKill) {
            system.runTimeout(() => {
                playersInGame.forEach(playerGame => {
                    Tellraw(playerGame, `${coloredThrowerName}  ${coloredThrowerName}`);
                });
                thrower.setGameMode("spectator");
                modifyScore("fear:deach", thrower.name, 1);
                updateTagData(thrower, "deaths", 1, true);
            }, 10);

        }
        system.runTimeout(() => {
            victims.forEach(victim => {

                if (victim.name === throwerName) return; 
    
                const victimTeam = getPlayerTeam(victim);
                const coloredVictimName = getColoredName(victim.name, victimTeam);
    
                playersInGame.forEach(playerGame => {
                    Tellraw(playerGame, `${coloredThrowerName}  ${coloredVictimName}`);
                });
    
                victim.setGameMode("spectator");
    
                Tellraw(thrower, `+300$ §a`, "reward.kill");
                modifyScore("fear:balance", thrower.name, 300);
                modifyScore("fear:kill", thrower.name, 1);
                modifyScore("fear:deach", victim.name, 1);
                updateTagData(thrower, "kills", 1, true);
                updateTagData(victim, "deaths", 1, true);
            });
        }, 10);


        tickCounter += 10;
        system.runTimeout(burnArea, 10);
    }

    burnArea();
}

