import { world, system } from "@minecraft/server";
import { getPlayerTeam, distance } from "./grenades.js";
import {  Tellraw, modifyScore, runcmd_at } from '../../other/utils.js';

const activeFlashGrenades = new Map();

world.beforeEvents.itemUse.subscribe(event => {
    const { itemStack, source } = event;

    if (itemStack.typeId === "addon:flash_grenade" && source.typeId === "minecraft:player") {
        const name = source.name;

        runcmd_at(name, `fill ~10~-10~10~-10~10~-10 air replace gb:muzzle_flash1`, true);
        runcmd_at(name, `fill ~10~-10~10~-10~10~-10 air replace gb:muzzle_flash2`, true);

        activeFlashGrenades.set(source.id, {
            name,
            team: getPlayerTeam(source),
            location: { ...source.location }
        });
    }
});

export function handleFlashGrenade(entity) {
    let thrower = getNearestThrower(entity.location);
    if (!thrower) return;

    system.run(function checkFall() {
        if (entity.getVelocity().y === 0) {
            system.runTimeout(() => explodeFlashGrenade(entity, thrower.name, thrower.team), 30);
        } else {
            system.run(() => checkFall());
        }
    });

    activeFlashGrenades.delete(thrower.id);
}

function getNearestThrower(grenadePos) {
    let nearest = null;
    let minDist = 4;

    for (let [id, thrower] of activeFlashGrenades.entries()) {
        let dist = distance(grenadePos, thrower.location);
        if (dist < minDist) {
            minDist = dist;
            nearest = { id, ...thrower };
        }
    }
    return nearest;
}

export function explodeFlashGrenade(grenade, throwerName, throwerTeam) {
    const flashRadius = 10;
    const affectedPlayers = [];

    const dim = world.getDimension("overworld");

    for (const player of world.getAllPlayers()) {
        const playerTeam = getPlayerTeam(player);
        const dist = distance(grenade.location, player.location);

        if (dist <= flashRadius && (playerTeam !== throwerTeam || player.name === throwerName)) {
            if (isLookingAtEntity(player, grenade) && !isObstructed(player, grenade)) {
                const name = player.name;
                affectedPlayers.push(name);

                // Вспышка камеры
                runcmd_at(name, `camera @s fade time 0 6 2 color 255 255 255`, true);

                // Звук игроку
                dim.playSound("addon.flashbang", player.location);

                // Эффект ослепления через тег
                runcmd_at(name, `tag @s add game:in_flash`, true);

                // Удаление тега через 60 тиков (3 секунды)
                system.runTimeout(() => {
                    runcmd_at(name, `tag @s remove game:in_flash`, true);
                }, 60);
            }
        }
    }

    // Эффекты в мире
    dim.spawnParticle("addon:flash_grenade", grenade.location);
    dim.playSound("addon.flash_grenade_explode", grenade.location);

    grenade.remove();
}

function isLookingAtEntity(player, entity) {
    const viewDirection = player.getViewDirection();
    if (!viewDirection) return false;

    const playerPos = player.location;
    const entityPos = entity.location;

    let dx = entityPos.x - playerPos.x;
    let dy = entityPos.y - (playerPos.y + 1.62);
    let dz = entityPos.z - playerPos.z;

    let mag = Math.sqrt(dx * dx + dy * dy + dz * dz);
    dx /= mag;
    dy /= mag;
    dz /= mag;

    let dotProduct = dx * viewDirection.x + dy * viewDirection.y + dz * viewDirection.z;

    return dotProduct > 0.4;
}

function isObstructed(player, grenade) {
    const dimension = world.getDimension("overworld");
    const playerPos = player.location;
    const grenadePos = grenade.location;

    let dx = grenadePos.x - playerPos.x;
    let dy = (grenadePos.y + 0.5) - (playerPos.y + 1.62); 
    let dz = grenadePos.z - playerPos.z;
    
    let steps = Math.max(Math.abs(dx), Math.abs(dy), Math.abs(dz)) * 2;
    let stepX = dx / steps;
    let stepY = dy / steps;
    let stepZ = dz / steps;

    let x = playerPos.x;
    let y = playerPos.y + 1.62;
    let z = playerPos.z;

    for (let i = 0; i < steps; i++) {
        x += stepX;
        y += stepY;
        z += stepZ;

        let block = dimension.getBlock({ x: Math.floor(x), y: Math.floor(y), z: Math.floor(z) });

        if (block && !block.isAir) {
            return true; 
        }
    }

    return false;
}
