import { world, system } from "@minecraft/server";
import { 
    getPlayerTeam, getColoredName, 
    getPlayerByName, distance 
} from "./grenades";
import {  Tellraw, modifyScore, runcmd_at, runcmd } from '../../other/utils';
import {  checkVictoryConditions } from '../../game/vinerRaund.js';

const activeHEGrenades = new Map();

world.beforeEvents.itemUse.subscribe(event => {
    const { itemStack, source } = event;
    
    if (itemStack.typeId === "addon:he_grenade" && source.typeId === "minecraft:player") {
        activeHEGrenades.set(source.id, {
            name: source.name,
            team: getPlayerTeam(source),
            location: { ...source.location }
        });
    }
});

export function handleHEGrenade(entity) {
    let thrower = getNearestThrower(entity.location);
    if (!thrower) return;

    system.run(function checkFall() {
        if (entity.getVelocity().y === 0) {
            system.runTimeout(() => explodeHEGrenade(entity, thrower.name, thrower.team), 30);
        } else {
            system.run(() => checkFall());
        }
    });

    activeHEGrenades.delete(thrower.id);
}

function getNearestThrower(grenadePos) {
    let nearest = null;
    let minDist = 4;

    for (let [id, thrower] of activeHEGrenades.entries()) {
        let dist = distance(grenadePos, thrower.location);
        if (dist < minDist) {
            minDist = dist;
            nearest = { id, ...thrower };
        }
    }
    return nearest;
}

function explodeHEGrenade(grenade, throwerName, throwerTeam) {
    if (!grenade || grenade.isRemoved) return;

    const pos = grenade.location;
    const formattedPos = `${Math.round(pos.x)} ${Math.round(pos.y)} ${Math.round(pos.z)}`;

    runcmd(`execute positioned ${formattedPos} as @e[r=4,type=addon:smoke_grenade] at @s run stopsound @a[r=20] addon.smoke_grenade_explode`, true);
    runcmd(`execute positioned ${formattedPos} as @e[r=4,type=addon:smoke_grenade] at @s run tag @s add game:smokFalse`, true);


    const explosionRadius = 4;
    const maxDamage = 30;
    let victims = [];
    let thrower = getPlayerByName(throwerName);
    if (!thrower) return; 

    let selfKill = false;
    const coloredThrowerName = getColoredName(throwerName, throwerTeam);

    for (let player of world.getAllPlayers()) {
        let playerTeam = getPlayerTeam(player);
        let dist = distance(grenade.location, player.location);

        if (dist <= explosionRadius) {
            let damage = Math.max(5, maxDamage - (dist * 5));
            player.applyDamage(damage);

            if (player.getComponent("minecraft:health")?.currentValue <= 0) {
                victims.push(player);
                if (player.name === throwerName) selfKill = true;
            }
        }
    }

    const playersInGame = world.getPlayers().filter(playerGame => playerGame.hasTag("fear:game"));

    world.getDimension("overworld").spawnParticle("addon:he_grenade", grenade.location);
    world.getDimension("overworld").playSound("addon.he_grenade_explode", grenade.location);

    if (selfKill) {
        system.runTimeout(() => {
            playersInGame.forEach(playerGame => {
                Tellraw(playerGame, `${coloredThrowerName}  ${coloredThrowerName}`);
            });
            thrower.setGameMode("spectator");
            modifyScore("fear:deach", thrower.name, 1);
        }, 10);

    }

    system.runTimeout(() => {
        victims.forEach(victim => {
            if (victim.name === throwerName) return; 
    
            const victimTeam = getPlayerTeam(victim);
            const coloredVictimName = getColoredName(victim.name, victimTeam);
    
            playersInGame.forEach(playerGame => {
                Tellraw(playerGame, `${coloredThrowerName}  ${coloredVictimName}`);
            });
    
            victim.setGameMode("spectator");
    
            Tellraw(thrower, `+300$ §a`, "reward.kill");
            modifyScore("fear:balance", thrower.name, 300);
    
            modifyScore("fear:kill", thrower.name, 1);
            modifyScore("fear:deach", victim.name, 1);
        });
    }, 10);


    grenade.remove();
}





