import { world, system, Player, EntityHealthComponent, EntityComponentTypes, ItemStack } from "@minecraft/server";
import { ActionFormData, MessageFormData, ModalFormData, uiManager } from "@minecraft/server-ui";
// import { 
//     getScore, eventEntity,
//     getPlayerCountWithTag, 
//     playSound, Tellraw, 
//     ActionBar, cloceUI,
//     removeTagsByPrefix, 
//     getPlayerNamesWithTag, 
//     ReplaceItemHotbar, 
//     getHealthPercentage, 
//     ReplaceItem, getPlayerByName,
//     hasItemInInventory,
//     incrementScore,
//     modifyScore,
//     getRandomInRange,
//     cloceForm

// } from '../other/utils';

export function clearNonPlayerTags(player) {
    const tags = player.getTags();

    for (const tag of tags) {
        if (!tag.startsWith("PlayerTag") && !tag.startsWith("fearcube") && !tag.startsWith("admin")) {
            system.run(() => {
                player.removeTag(tag);
            });
        }
    }
}



export function runcmd(command, need_log = false) {
    system.run(() => {
        try {
            const result = world.getDimension("overworld").runCommand(command);
            if (!result.success && need_log) {
            }
        } catch (error) {
            if (need_log) {
                console.error(`CMD: ${command}\nError: ${error}`);
            }
        }
    });
}

export function runcmd_at(name, command, need_log = false) {
    const target = isNotSelector(name) ? `"${name}"` : name;
    runcmd(`execute as ${target} at @s run ${command}`, need_log);
}

function isNotSelector (text){
    return text.indexOf('@') == -1
}

export function clearBombFromInventory(player) {
    const inventoryComp = player.getComponent("minecraft:inventory");
    if (!inventoryComp) return;

    const container = inventoryComp.container;

    for (let i = 0; i < container.size; i++) {
        const item = container.getItem(i);
        if (item && item.typeId === "fear:bomb") {
            container.setItem(i, undefined); // Удаляем предмет
        }
    }
}

// const player = getPlayerByName(playerName);
export function getPlayerByName(playerName) {
    if (!playerName || typeof playerName !== "string") {
        console.error("Некорректное имя игрока.");
        return null;
    }

    const trimmedName = playerName.trim();
    const player = world.getPlayers().find(p => p.name.trim() === trimmedName);

    if (!player) {
        console.warn(`Игрок с именем "${trimmedName}" не найден.`);
        return null;
    }

    return player;
}


export function eventEntity(type, event) {
    try {
        const entities = world.getEntities({ type });

        if (!entities || entities.length === 0) {
            return;
        }

        for (const entity of entities) {
            entity.triggerEvent(event);
        }

    } catch (error) {
        console.error(`Ошибка при выполнении ивента "${event}" для сущностей типа "${type}":`, error);
    }
}


// const countInLobby = getPlayerCountWithTag("fear:lobby");
export function getPlayerCountWithTag(tag) {
    return world.getPlayers().filter(player => player.hasTag(tag)).length;
}

export function cloceForm(player) {
    system.run(() => {
        uiManager.closeAllForms(player);
    });
}

export function cloceUI(player) {
    if (!player || typeof player.name !== "string") return;

    const name = player.name;
    const titles = [
        "scoreboard",
        "bossbar",
        "upper_right_corner",
        "linen_plate",
        "timer_top_midle",
        "big_actionbar",
        "top_subtitle2",
        "top_subtitle3",
        "top_subtitle4"
    ];

    let index = 0;

    function showNextTitle() {
        if (index >= titles.length) return;

        const title = titles[index];
        runcmd_at(name, `title @s title ${title}`, true);

        index++;
        system.runTimeout(showNextTitle, 1);
    }

    showNextTitle();
}




// const sound = getRandomInRange(1, 2);
export function getRandomInRange(min, max) {
    return Math.floor(Math.random() * (max - min + 1)) + min;
}


// modifyScore("fear:balance", "playerName", -10); // Уменьшает очки на 10
// modifyScore("fear:balance", "playerName", 20);  // Увеличивает очки на 20
export function modifyScore(objectiveName, targetName, increment = 1) {
    system.run(() => {
        try {
            const objective = world.scoreboard.getObjective(objectiveName);
            if (!objective) {
                console.error(`Objective "${objectiveName}" не найдено.`);
                return NaN;
            }

            let participant = objective.getParticipants().find(p => p.displayName === targetName);

            if (!participant) {
                participant = objective.addParticipant(targetName);
            }

            let currentScore = participant ? objective.getScore(participant) : 0;
            const newScore = currentScore + increment;

            objective.setScore(participant, newScore);
            return newScore;
        } catch (error) {
            console.error(`Ошибка при работе с scoreboard:`, error);
            return NaN;
        }
    });
}

export function setScore(objectiveName, targetName, value) {
    system.run(() => {
        try {
            const objective = world.scoreboard.getObjective(objectiveName);
            if (!objective) {
                console.error(`Objective "${objectiveName}" не найдено.`);
                return NaN;
            }

            let participant = objective.getParticipants().find(p => p.displayName === targetName);

            if (!participant) {
                participant = objective.addParticipant(targetName);
            }

            objective.setScore(participant, value);
            return value;
        } catch (error) {
            console.error(`Ошибка при работе с scoreboard:`, error);
            return NaN;
        }
    });
}

export function getScore(objectiveName, target, useZero = false) {
    try {
        const objective = world.scoreboard.getObjective(objectiveName);
        if (!objective) {
            console.error(`Objective "${objectiveName}" не найдено.`);
            return useZero ? 0 : NaN;
        }

        if (typeof target !== 'string') {
            target = target.name || target.nameTag;
        }

        const participant = objective.getParticipants().find(p => p.displayName === target);

        if (!participant) {
            return useZero ? 0 : NaN;
        }

        return objective.getScore(participant);
    } catch (error) {
        console.error(`Ошибка при получении счёта:`, error);
        return useZero ? 0 : NaN;
    }
}

export function incrementScore(objectiveName, targetName, increment = 1) {
    try {
        const objective = world.scoreboard.getObjective(objectiveName);
        if (!objective) {
            console.error(`Objective "${objectiveName}" не найдено.`);
            return NaN;
        }

        let participant = objective.getParticipants().find(p => p.displayName === targetName);

        if (!participant) {
            participant = objective.addParticipant(targetName);
        }

        let currentScore = participant ? objective.getScore(participant) : 0;
        const newScore = currentScore + increment;

        objective.setScore(participant, newScore);
        return newScore;
    } catch (error) {
        console.error(`Ошибка при работе с scoreboard:`, error);
        return NaN;
    }
}




// const hasApple = hasItemInInventory(player, "minecraft:apple", 5);
export function hasItemInInventory(player, itemId, quantity = 1) {
    const inventory = player.getComponent("minecraft:inventory")?.container;

    if (!inventory) {
        console.error("Инвентарь недоступен у игрока.");
        return false;
    }

    let itemCount = 0;

    for (let i = 0; i < inventory.size; i++) {
        const itemStack = inventory.getItem(i);
        if (itemStack && itemStack.typeId === itemId) {
            itemCount += itemStack.amount;

            if (itemCount >= quantity) {
                return true; // Если найдено достаточное количество, возвращаем true
            }
        }
    }

    return false; // Если недостаточно предметов или они отсутствуют
}


// const terroristNames = getPlayerNamesWithTag("game:terrorist");
export function getPlayerNamesWithTag(tag) {
    return world.getPlayers()
        .filter(player => player.hasTag(tag)) // Фильтруем игроков с указанным тегом
        .map(player => player.name); // Возвращаем только их имена
}



//  playSound(player, "fear:fear_cube.hurt", 1, 1);
export function playSound(player, soundKey, volume = 1, pitch = 1) {
    const playerName = player.name;

    // Остановить звук
    runcmd_at(playerName, `stopsound @s ${soundKey}`, true);

    // Воспроизвести звук в следующем тике (гарантированный порядок)
    system.runTimeout(() => {
        runcmd_at(playerName, `playsound ${soundKey} @s ~~~ ${volume} ${pitch}`, true);
    }, 1);
}



// Tellraw(player, "Текст1 ", "Ключ перевода1", " Текст2 ", "Ключ перевода2");
export function Tellraw(player, ...components) {
    const name = player.name;
    const rawText = [];

    for (let i = 0; i < components.length; i++) {
        if (i % 2 === 0) {
            rawText.push({ text: components[i] });
        } else {
            rawText.push({ translate: components[i] });
        }
    }

    const json = JSON.stringify({ rawtext: rawText });
    const command = `tellraw @s ${json}`;

    runcmd_at(name, command, true);
}


// ActionBar(player, "Текст1 ", "Ключ перевода1", " Текст2 ", "Ключ перевода2");
export function ActionBar(player, ...components) {
    const name = player.name;
    const rawText = [];

    for (let i = 0; i < components.length; i++) {
        if (i % 2 === 0) {
            rawText.push({ text: components[i] });
        } else {
            rawText.push({ translate: components[i] });
        }
    }

    const json = JSON.stringify({ rawtext: rawText });
    const command = `titleraw @s actionbar ${json}`;

    runcmd_at(name, command, true);
}


// removeTagsByPrefix(player, "game:");
export function removeTagsByPrefix(player, prefix) {
    const tags = player.getTags(); 
    const matchingTags = tags.filter(tag => tag.startsWith(prefix)); 

    matchingTags.forEach(tag => {
        system.run(() => {
            player.removeTag(tag); 
        });
    });
}


// ReplaceItem(player, itemId, slotType, slotIndex, count = 1, lockSlot = false)
export function ReplaceItem(player, itemId, slotType, slotIndex, count = 1, lockSlot = false) {
    const name = player.name;
    let itemLockJSON = "";

    if (lockSlot) {
        itemLockJSON = `{"minecraft:item_lock":{"mode":"lock_in_slot"}}`;
    }

    const command = `replaceitem entity @s ${slotType} ${slotIndex} ${itemId} ${count} 0 ${itemLockJSON}`;

    runcmd_at(name, command, true);
}






// ReplaceItemHotbar(player, "minecraft:apple", 1, 3, false); 1=slot 3=cound
export function ReplaceItemHotbar(player, itemId, slot, count = 1, lockSlot = false) {
    const name = player.name;
    let itemLockJSON = "";

    if (lockSlot) {
        itemLockJSON = `{"minecraft:item_lock":{"mode":"lock_in_slot"}}`;
    }

    const command = `replaceitem entity @s slot.hotbar ${slot} ${itemId} ${count} 0 ${itemLockJSON}`;

    runcmd_at(name, command, true);
}


// const healthPercentage = getHealthPercentage(player);
export function getHealthPercentage(player) {
    try {
        if (player.getGameMode() === "spectator") {
            console.warn(`Игрок ${player.name} находится в режиме зрителя.`);
            return "N/A";
        }

        const healthComponent = player.getComponent(EntityComponentTypes.Health);
        if (!healthComponent) {
            console.error(`Компонент здоровья недоступен для игрока ${player.name}`);
            return "N/A";
        }

        const currentHealth = healthComponent.currentValue;
        const maxHealth = healthComponent.defaultValue;

        if (currentHealth === undefined || maxHealth === undefined || maxHealth === 0) {
            console.error(`Некорректные значения здоровья для игрока ${player.name}`);
            return "N/A";
        }

        const healthPercent = Math.floor((currentHealth / maxHealth) * 100);
        return `${healthPercent}%`;
    } catch (error) {
        console.error(`Ошибка получения здоровья для игрока ${player.name}: ${error.message}`);
        return "N/A";
    }
}